Integer

Struct Integer 

pub struct Integer(/* private fields */);
Big integer type used in this crate

Implementations



impl Integer

pub fn to_bytes_lsf(&self) -> Vec<u8>

Converts to bytes, with bytes representing least significant base256 digits appearing first. Discards the sign

Example
let x = Integer::from(0x11223344);
assert_eq!(x.to_bytes_lsf(), vec![0x44, 0x33, 0x22, 0x11]);

pub fn to_bytes_msf(&self) -> Vec<u8>

Converts to bytes, with bytes representing most significant base256 digits appearing first. Discards the sign

Example
let x = Integer::from(0x11223344);
assert_eq!(x.to_bytes_msf(), vec![0x11, 0x22, 0x33, 0x44]);

pub fn to_bytes_msf_signed(&self) -> (Vec<u8>, Sign)

Converts to bytes, with bytes representing most significant base256 digits appearing first

Example
let x = Integer::from(-0x11223344);
assert_eq!(x.to_bytes_msf_signed(), (vec![0x11, 0x22, 0x33, 0x44],
Sign::Negative));

pub fn from_bytes_msf(bytes: &[u8]) -> Integer

Converts bytes to Integer. Inverse of Integer::to_bytes_msf

pub fn from_bytes_msf_signed(bytes: &[u8], sign: Sign) -> Integer

Converts bytes to Integer. Inverse of Integer::to_bytes_msf_signed

pub fn to_str_radix(&self, radix: u16) -> String

Returns a string representation of the number for the specified radix

pub fn from_str_radix(s: &str, radix: u16) -> Option<Integer>

Parses the integer using the given radix

pub fn to_num_bigint(self) -> BigInt

Convert the number to the underlying backend representation

pub fn from_num_bigint(x: BigInt) -> Integer

Convert the number from the underlying backend representation



impl Integer

pub fn one() -> Integer

pub fn zero() -> Integer

pub fn is_one(&self) -> bool

pub fn is_even(&self) -> bool

pub fn abs(self) -> Integer

pub fn cmp_abs(&self, other: &Integer) -> Ordering

pub fn lcm_ref(&self, other: &Integer) -> Integer

pub fn gcd_ref(&self, other: &Integer) -> Integer

pub fn cmp0(&self) -> Ordering

pub fn sign(&self) -> Sign

pub fn pow_mod(self, exponent: &Integer, modulo: &Integer) -> Option<Integer>

pub fn pow_mod_ref( &self, exponent: &Integer, modulo: &Integer, ) -> Option<Integer>

pub fn u_pow_u(base: u32, exponent: u32) -> Integer

pub fn square(self) -> Integer

pub fn square_ref(&self) -> Integer

pub fn sqrt(self) -> Option<Integer>

pub fn sqrt_ref(&self) -> Option<Integer>

pub fn modulo(self, divisor: &Integer) -> Integer

pub fn modulo_ref(&self, divisor: &Integer) -> Integer

pub fn modulo_mut(&mut self, divisor: &Integer) -> &mut Integer

pub fn mod_u(&self, modulo: u32) -> u32

pub fn significant_bits(&self) -> u64

pub fn significant_dwords(&self) -> usize

pub fn invert(self, modulo: &Integer) -> Option<Integer>

pub fn invert_ref(&self, modulo: &Integer) -> Option<Integer>

pub fn set_bit(&mut self, index: u32, value: bool) -> &mut Integer

pub fn random_below(self, rng: &mut impl RngCore) -> Integer

pub fn random_below_ref(&self, rng: &mut impl RngCore) -> Integer

pub fn random_bits(bits: u32, rng: &mut impl RngCore) -> Integer

pub fn random_bits_signed(bits: u32, rng: &mut impl RngCore) -> Integer

pub fn assign_random_below( &mut self, modulo: &Integer, rng: &mut impl RngCore, ) -> &mut Integer

pub fn assign_random_bits( &mut self, bits: u32, rng: &mut impl RngCore, ) -> &mut Integer

pub fn is_probably_prime(&self, _reps: u32, rng: &mut impl RngCore) -> IsPrime

pub fn generate_prime(rng: &mut impl RngCore, bit_size: u32) -> Integer

pub fn jacobi(&self, n: &Integer) -> i32

Compute jacobi symbol of a over n

Requires odd n >= 3. If it doesn’t hold, function panics if debug asserts are enabled, or returns 0 otherwise.

Implementation is taken from Handbook of Applied cryptography, p. 73, Algorithm 2.149

pub fn combine( &self, l: &Integer, le: &Integer, r: &Integer, re: &Integer, ) -> Option<Integer>

Compute l^le * r^re modulo self



impl Integer

pub fn in_mult_group_of(&self, n: &Integer) -> bool

Checks that self is in Z*n

pub fn abs_in_mult_group_of(&self, n: &Integer) -> bool

Checks that abs(self) is in Z*n

pub fn sample_in_mult_group_of(rng: &mut impl RngCore, n: &Integer) -> Integer

Samples x in Z*_n

pub fn sample_pm_in_mult_group_of( rng: &mut impl RngCore, n: &Integer, ) -> Integer

Samples x such that abs(x) is in Z*_n

pub fn generate_safe_prime(rng: &mut impl RngCore, bits: u32) -> Integer

Generates a random safe prime